/**************************************************************************************************
  Real Time Clock driver.
**************************************************************************************************/

// Include Files
#include "RTC.h"
#include "Buffers.h"
#include "USB_Driver.h"
#include "LED.h"



// Variables
TTime MyTime;                          // Date/time structure area

unsigned long RTC_Time;
unsigned long RTC_Date;



/**************************************************************************************************
  RTC Initialize */
void RTC_Initialize(void) {
     
     RCC_APB1ENR.PWREN = 1;            // Enable power interface clock
     
     if (RCC_BDCR.RTCEN == 0) {        // if RTC is disabled ...
         PWR_CR.DBP = 1;               // Allow access to RTC
         RTC_WPR = 0xCA;               // Unlock write protection
         RTC_WPR = 0x53;               // Unlock write protection

         RCC_BDCR = 0x00010000;        // Make it possible to change clock source
         RCC_BDCR = 0x00008101;        // Set RTCEN, LSE, LSEON
     }

     //LED_Set_Red();  // Diagnostic
     while(RTC_ISR.RSF != 1) {}        // Wait for RTC APB registers synchronisation
     //LED_Clr_Red();  // Diagnostic

     //LED_Set_Yellow();  // Diagnostic
     while(RCC_BDCR.LSERDY != 1) {};   // Wait till LSE is ready
     //LED_Clr_yellow();  // Diagnostic

     RTC_WPR = 0xFF;                   // Set write protection register lock
}

/**************************************************************************************************  
  RTC write */
void RTC_Write(TTime *RTCC_Time) {
     char *BufNum;
     unsigned long temp;

     PWR_CR.DBP = 1;                   // Enable access to RTC, RTC backup regs, SRAM
     RTC_WPR = 0xCA;                   // Unlock write protection
     RTC_WPR = 0x53;                   // Unlock write protection

     RTC_ISR = 0x00000080;             // Set initialization mode (bit 7 of RTC_ISR)
     
     //LED_Set_Red();  // Diagnostic
     while(RTC_ISR.INITF != 1) {};     // Wait for INITF (bit 6 of RTC_ISR) to come on
     //LED_Clr_Red();  // Diagnostic

     RTC_PRERBITS.PREDIV_S = 0xFF;     // Set synchronous prescaler factor to 256
     RTC_PRERBITS.PREDIV_A = 0x7F;     // Set asynchronous division factor to 128

     // Set the date register
     // Those engineers at ST need a good talking to! Haven't they learned anything from the
     // Y2K fiasco? They reserved a nice space for the century but have not implemented it.
     temp = (unsigned long)Dec2BCD16(RTCC_Time->year) << 16;
     temp |= (unsigned long)RTCC_Time->weekday << 13;
     temp |= (unsigned long)Dec2BCD(RTCC_Time->month) << 8;
     temp |= (unsigned long)Dec2Bcd(RTCC_Time->day);
     
     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"%08LX\r\n",temp); // Display assembled long word
     USB_PutQueue(BufNum);
     
     RTC_DR = temp;                    // Set date register in RTC
     
     // Set the time register
     //temp += (unsigned long)(RTCC_Time->ampm) << 22;  // Not required in 24hr mode
     temp = (unsigned long)Dec2Bcd(RTCC_Time->hours) << 16;
     temp |= (unsigned long)Dec2BCD(RTCC_Time->minutes) << 8;
     temp |= (unsigned long)Dec2BCD(RTCC_Time->seconds);
     
     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"%08LX\r\n",temp); // Display assembled long word
     USB_PutQueue(BufNum);
     
     RTC_TR = temp;                    // Set Time register in RTC

     RTC_CRbits.FMT = 0;               // Set FMT 24H format
     RTC_CRbits.WCKSEL = 0;            // Set RTC/16 clock

     RTC_ISR = 0x00000000;             // Exit initialization mode

     RTC_WPR = 0xFF;                   // Enable the write protection for RTC registers
}

/**************************************************************************************************
  RTC Read */
void RTC_Read(TTime *RTCC_Time) {
     char *BufNum;
     
     RTC_Time = RTC_TR;              // Read the time register
     RTC_Date = RTC_DR;              // Read the date register

     // Extract the date fields
     RTCC_Time->year = Bcd2Dec16((unsigned)((RTC_Date & 0xFFFF0000) >> 16));
     RTCC_Time->weekday = Bcd2Dec((short)((RTC_Date & 0x0000E000) >> 13));
     RTCC_Time->month = Bcd2Dec((short)((RTC_Date & 0x00001F00) >> 8));
     RTCC_Time->day = Bcd2Dec((short)(RTC_Date & 0x0000003F));
     
     // Extract the Time fields
     RTCC_Time->hours = Bcd2Dec((short)((RTC_Time & 0x003F0000) >> 16));
     RTCC_Time->minutes = Bcd2Dec((short)((RTC_Time & 0x00007F00) >> 8));
     RTCC_Time->seconds = Bcd2Dec((short)(RTC_Time & 0x0000007F));
     
     BufNum = GetBuffer();     // Get a free buffer
     sprintf(BufNum,"%04d/%02d/%02d %02d:%02d:%02d %d\r\n",MyTime.year,MyTime.month,MyTime.day,MyTime.hours,MyTime.minutes,MyTime.seconds,MyTime.weekday);
     USB_PutQueue(BufNum);   // Display the buffer
}

/**************************************************************************************************
  RTC Write Backup Registers */
void RTC_WriteSRAM(unsigned short Reg, unsigned long Value) {

     switch (Reg) {
         case 0: RTC_BKP0R = Value; break;
         case 1: RTC_BKP1R = Value; break;
         case 2: RTC_BKP2R = Value; break;
         case 3: RTC_BKP3R = Value; break;
         case 4: RTC_BKP4R = Value; break;
         case 5: RTC_BKP5R = Value; break;
         case 6: RTC_BKP6R = Value; break;
         case 7: RTC_BKP7R = Value; break;
         case 8: RTC_BKP8R = Value; break;
         case 9: RTC_BKP9R = Value; break;
         case 10: RTC_BKP10R = Value; break;
         case 11: RTC_BKP11R = Value; break;
         case 12: RTC_BKP12R = Value; break;
         case 13: RTC_BKP13R = Value; break;
         case 14: RTC_BKP14R = Value; break;
         case 15: RTC_BKP15R = Value; break;
         case 16: RTC_BKP16R = Value; break;
         case 17: RTC_BKP17R = Value; break;
         case 18: RTC_BKP18R = Value; break;
         case 19: RTC_BKP19R = Value; break;
         default: USB_PutQueue("Invalid reg address\r\n"); break;
     }
}

/**************************************************************************************************
  RTC Read Backup Registers */
unsigned long RTC_ReadSRAM(unsigned short Reg) {
     unsigned long Temp;

     switch (Reg) {
         case 0: Temp = RTC_BKP0R; break;
         case 1: Temp = RTC_BKP1R; break;
         case 2: Temp = RTC_BKP2R; break;
         case 3: Temp = RTC_BKP3R; break;
         case 4: Temp = RTC_BKP4R; break;
         case 5: Temp = RTC_BKP5R; break;
         case 6: Temp = RTC_BKP6R; break;
         case 7: Temp = RTC_BKP7R; break;
         case 8: Temp = RTC_BKP8R; break;
         case 9: Temp = RTC_BKP9R; break;
         case 10: Temp = RTC_BKP10R; break;
         case 11: Temp = RTC_BKP11R; break;
         case 12: Temp = RTC_BKP12R; break;
         case 13: Temp = RTC_BKP13R; break;
         case 14: Temp = RTC_BKP14R; break;
         case 15: Temp = RTC_BKP15R; break;
         case 16: Temp = RTC_BKP16R; break;
         case 17: Temp = RTC_BKP17R; break;
         case 18: Temp = RTC_BKP18R; break;
         case 19: Temp = RTC_BKP19R; break;
         default: USB_PutQueue("Invalid reg address\r\n"); Temp = 0; break;
     }
     return Temp;
}

/**************************************************************************************************
  RTC Write Coarse Calibration Register.
  Cal bit 7 = 0  -  make clock faster
  Cal bit 7 = 1  -  make clock slower
  Cal bits 4:0   -  calibration value */
void RTC_WriteCCR(char Cal) {
     
     PWR_CR.DBP = 1;                   // Enable access to RTC, RTC backup regs, SRAM
     RTC_WPR = 0xCA;                   // Unlock write protection
     RTC_WPR = 0x53;                   // Unlock write protection
     RTC_ISR = 0x00000080;             // Set initialization mode (bit 7 of RTC_ISR)
     while(RTC_ISR.INITF != 1) {};     // Wait for INITF (bit 6 of RTC_ISR) to come on
     RTC_CALIBR = Cal;                 // Set calibration register with value supplied
     RTC_ISR = 0x00000000;             // Exit initialization mode
     RTC_WPR = 0xFF;                   // Set write protection register lock
}

/**************************************************************************************************
  RTC Read Coarse Calibration Register. */
void RTC_ReadCCR(void) {
     char *BufNum;
     
     BufNum = GetBuffer();
     sprintf(BufNum,"%04X\r\n",RTC_CALIBR);
     USB_PutQueue(BufNum);
}

/**************************************************************************************************
  RTC Read Initialization and Status Register. */
void RTC_ReadISR(void) {
     char *BufNum;

     BufNum = GetBuffer();
     sprintf(BufNum,"%08LX\r\n",RTC_ISR);
     USB_PutQueue(BufNum);
}