/**************************************************************************************************
   USB_Driver
   This module is used for communications with the user through HID Terminal.
**************************************************************************************************/
#include "USB_Driver.h"
#include "Buffers.h"
#include "LED.h"
#include "RTC.h"
#include "ADC_DAC.h"
#include "Timers.h"




// Variables
char readbuff[64];
char writebuff[64];

USB_TxStruct USB_Queue;        // Transmit buffer FIFO and input/output pointers

extern TTime MyTime;
extern unsigned short int Amp;



/**************************************************************************************************
   This is the USB interrupt handler */

void USBInterrupt() iv IVT_INT_OTG_FS ics ICS_AUTO {
     USB_Interrupt_Proc();
}

/**************************************************************************************************
   Initialization */
void USB_Init(void) {
     //USBIE_bit = 0;
     //IPC11bits.USBIP = 7;
   
     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
     //NVIC_IntEnable(IVT_INT_OTG_FS);  // Enable USB device interrupt
}

/**************************************************************************************************
   This routine initializes the transmit buffer FIFO in and out pointers */
void USB_TxQueueInit(void) {
     
     USB_Queue.in = 0;
     USB_Queue.out = 0;
}

/**************************************************************************************************
   This routine makes an entry into the transmit FIFO and updates the in pointer to the next
   available location. */
void USB_PutQueue(char *msg) {
     
     USB_Queue.q[USB_Queue.in++] = msg;
     if (USB_Queue.in > (USB_Queue_Size-1))
         USB_Queue.in = 0;
}

/**************************************************************************************************
   This routine checks the transmit buffer FIFO and, if it is not empty, sends the buffer to USB.
   If the buffer was successfully sent, the out pointer is updated to the next available location.
   */

// Call this function in main loop
void USB_CheckOut(void) {

     if (USB_Queue.in != USB_Queue.out) {          // Look for a pointer in the FIFO

          // This is a blocking function
          if (HID_Write(USB_Queue.q[USB_Queue.out],64) != 0) {
              USB_Queue.out++;
              if (USB_Queue.out > (USB_Queue_Size-1))  // If past the end of array
                  USB_Queue.out = 0;                   // set pointer to start of array
          }
     }
}

/**************************************************************************************************
   This routine checks USB for a receive message then decodes the first character for a command. */

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;
     unsigned int Work;
     unsigned long BkpReg;

     if (HID_Read() != 0) {
          switch (readbuff[0]) {
               
               case 'a':  // Return a canned message that indicates everything is working OK
                   USB_PutQueue("Command a received\r\n");
                   break;
               case 'b':  // Set the yellow LED
                   LED_Set_Yellow();
                   USB_PutQueue("Yellow set\r\n");
                   break;
               case 'c':  // Clear the yellow LED
                   LED_Clr_Yellow();
                   USB_PutQueue("Yellow cleared\r\n");
                   break;
               case 'd':  // Set the red LED
                   LED_Set_Red();
                   USB_PutQueue("Red set\r\n");
                   break;
               case 'e':  // Clear the red LED
                   LED_Clr_Red();
                   USB_PutQueue("Red cleared\r\n");
                   break;
               case 'f':  // Set RTC from data in message
                   //  All data bytes are ASCII characters in the range of 0 to 9.
                   //  byte: 0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20
                   //  char: f  , YR YR  , WD  , MO MO  , MD MD  ,  h  h  ,  m  m  ,  s  s
                   //  e.g. - f,14,4,01,02,18,44,36
                   Work  = readbuff[2] & 0x0F;    // Year
                   Work <<= 4;
                   Work |= readbuff[3] & 0x0F;
                   MyTime.year = Bcd2Dec16(Work);

                   Work  = readbuff[5] & 0x0F;    // Week day
                   MyTime.weekday = Work;

                   Work  = readbuff[7] & 0x0F;    // Month
                   Work <<= 4;
                   Work |= readbuff[8] & 0x0F;
                   MyTime.month = Bcd2Dec(Work);

                   Work  = readbuff[10] & 0x0F;   // Day
                   Work <<= 4;
                   Work |= readbuff[11] & 0x0F;
                   MyTime.day = Bcd2Dec(Work);

                   Work  = readbuff[13] & 0x0F;   // Hour
                   Work <<= 4;
                   Work |= readbuff[14] & 0x0F;
                   MyTime.hours = Bcd2Dec(Work);

                   Work  = readbuff[16] & 0x0F;   // Minute
                   Work <<= 4;
                   Work |= readbuff[17] & 0x0F;
                   MyTime.minutes = Bcd2Dec(Work);

                   Work  = readbuff[19] & 0x0F;   // Second
                   Work <<= 4;
                   Work |= readbuff[20] & 0x0F;
                   MyTime.seconds = Bcd2Dec(Work);

                   BufNum = GetBuffer();     // Get a free buffer
                   sprintf(BufNum,"%04d/%02d/%02d %02d:%02d:%02d %d\r\n",MyTime.year,MyTime.month,MyTime.day,MyTime.hours,MyTime.minutes,MyTime.seconds,MyTime.weekday);
                   USB_PutQueue(BufNum);    // Display the buffer
                   
                   RTC_Write(&MyTime);  // Set the hardware Real Time Clock
                   break;
               case 'g':  // Display the RTC
                   RTC_Read(&MyTime);
                   break;
               case 'h':  // Set RTC RAM, read and display contents
                   RTC_WriteSRAM(0,0xAAAA5555);
                   RTC_WriteSRAM(1,0x5555AAAA);
                   
                   BkpReg = RTC_ReadSRAM(0);
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Addr0: %08LX\r\n",BkpReg);
                   USB_PutQueue(BufNum);
                   
                   BkpReg = RTC_ReadSRAM(1);
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Addr1: %08LX\r\n",BkpReg);
                   USB_PutQueue(BufNum);
                   break;
               case 'i':  // Display the Initialization and Status Register
                   RTC_ReadISR();
                   break;
               case 'j':  // Start DAC1 to produce 60Hz Sine wave on PA4
                   DAC1_Initialize();
                   Timer6_Initialize();
                   break;
               case 'k':  // Raise the amplitude of the Sine wave
                   if (Amp < 255)
                       Amp++;
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Amp: %02X\r\n",Amp);
                   USB_PutQueue(BufNum);
                   break;
               case 'l':  // Lower the amplitude of the Sine wave
                   if (Amp > 0)
                       Amp--;
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Amp: %02X\r\n",Amp);
                   USB_PutQueue(BufNum);
                   break;
               case 'm':
                   break;
               case 'n':
                   break;
               case 'o':
                   break;
               case 'p':
                   break;
               case 'q':
                   break;
               case 'r':
                   break;
               case 's':
                   break;
               case 't':
                   break;
               case 'u':
                   break;
               case 'v':
                   break;
               case 'w':
                   break;
               case 'x':
                   break;
               case 'y':
                   break;
               case 'z':
                   break;
               
               // Etc.
              
               default:
                   USB_PutQueue("Unknown command: ");
                   
                   BufNum = GetBuffer();         // Get a free buffer
                   (void)sprintf(BufNum,"%c\r\n",readbuff[0]);
                   USB_PutQueue(BufNum);
                   break;
          }
     }
}